@@ -10,6 +10,10 @@ import re
 import tarfile
 
 from utils import create_sparse_tempfile, create_lio_device, delete_lio_device, get_avail_locales, requires_locales, run_command, read_file, TestTags, tag_test
+
+import gi
+gi.require_version("BlockDev", "2.0")
+
 from gi.repository import BlockDev, GLib
 
 PASSWD = "myshinylittlepassword"
@@ -575,7 +579,7 @@ class CryptoTestGetMetadataSize(CryptoTestCase):
         if m is None:
             self.fail("Failed to get LUKS 2 offset information from %s:\n%s %s" % (self.loop_dev, out, err))
         offset = int(m.group(1))
-        self.assertEquals(meta_size, offset, "LUKS 2 metadata sizes differ")
+        self.assertEqual(meta_size, offset, "LUKS 2 metadata sizes differ")
 
     @tag_test(TestTags.SLOW)
     def test_luks_get_metadata_size(self):
@@ -596,7 +600,7 @@ class CryptoTestGetMetadataSize(CryptoTestCase):
         # offset value is in 512B blocks; we need to multiply to get the real metadata size
         offset = int(m.group(1)) * 512
 
-        self.assertEquals(meta_size, offset, "LUKS metadata sizes differ")
+        self.assertEqual(meta_size, offset, "LUKS metadata sizes differ")
 
 class CryptoTestLuksOpenRW(CryptoTestCase):
     def _luks_open_rw(self, create_fn):
