@@ -10,7 +10,10 @@ import re
 import six
 import overrides_hack
 
-from distutils.version import LooseVersion
+from packaging.version import Version
+
+import gi
+gi.require_version("BlockDev", "2.0")
 
 from gi.repository import BlockDev, GLib
 
@@ -41,7 +44,7 @@ def _get_dosfstools_version():
     m = re.search(r"mkfs\.fat ([\d\.]+)", out)
     if not m or len(m.groups()) != 1:
         raise RuntimeError("Failed to determine dosfstools version from: %s" % out)
-    return LooseVersion(m.groups()[0])
+    return Version(m.groups()[0])
 
 
 def _get_xfs_version():
@@ -49,7 +52,7 @@ def _get_xfs_version():
     m = re.search(r"mkfs\.xfs version ([\d\.]+)", out)
     if not m or len(m.groups()) != 1:
         raise RuntimeError("Failed to determine xfsprogs version from: %s" % out)
-    return LooseVersion(m.groups()[0])
+    return Version(m.groups()[0])
 
 
 class FSTestCase(unittest.TestCase):
@@ -76,6 +79,15 @@ class FSTestCase(unittest.TestCase):
         except:
             cls.ntfs_avail = False
 
+        try:
+            cls.exfat_avail = BlockDev.fs_is_tech_avail(BlockDev.FSTech.EXFAT,
+                                                        BlockDev.FSTechMode.MKFS |
+                                                        BlockDev.FSTechMode.REPAIR |
+                                                        BlockDev.FSTechMode.CHECK |
+                                                        BlockDev.FSTechMode.SET_LABEL)
+        except Exception :
+            cls.exfat_avail = False
+
     def setUp(self):
         self.addCleanup(self._clean_up)
         self.dev_file = utils.create_sparse_tempfile("fs_test", self._loop_size)
@@ -91,7 +103,7 @@ class FSTestCase(unittest.TestCase):
 
         self.mount_dir = tempfile.mkdtemp(prefix="libblockdev.", suffix="ext4_test")
 
-        if self._vfat_version <= LooseVersion("4.1"):
+        if self._vfat_version <= Version("4.1"):
             self._mkfs_options = None
         else:
             self._mkfs_options = [BlockDev.ExtraArg.new("--mbr=n", "")]
@@ -134,7 +146,7 @@ class TestGenericWipe(FSTestCase):
         with self.assertRaises(GLib.GError):
             BlockDev.fs_wipe("/non/existing/device", True)
 
-        ret = run("pvcreate -ff -y %s >/dev/null 2>&1" % self.loop_dev)
+        ret = run("pvcreate -ff -y %s --config \"devices {use_devicesfile = 0}\" >/dev/null 2>&1" % self.loop_dev)
         self.assertEqual(ret, 0)
 
         succ = BlockDev.fs_wipe(self.loop_dev, True)
@@ -142,7 +154,7 @@ class TestGenericWipe(FSTestCase):
 
         # now test the same multiple times in a row
         for i in range(10):
-            ret = run("pvcreate -ff -y %s >/dev/null 2>&1" % self.loop_dev)
+            ret = run("pvcreate -ff -y %s --config \"devices {use_devicesfile = 0}\" >/dev/null 2>&1" % self.loop_dev)
             self.assertEqual(ret, 0)
 
             succ = BlockDev.fs_wipe(self.loop_dev, True)
@@ -150,7 +162,7 @@ class TestGenericWipe(FSTestCase):
 
         # vfat has multiple signatures on the device so it allows us to test the
         # 'all' argument of fs_wipe()
-        if self._vfat_version >= LooseVersion("4.2"):
+        if self._vfat_version >= Version("4.2"):
             ret = utils.run("mkfs.vfat -I %s >/dev/null 2>&1 --mbr=n" % self.loop_dev)
         else:
             ret = utils.run("mkfs.vfat -I %s >/dev/null 2>&1" % self.loop_dev)
@@ -175,7 +187,7 @@ class TestGenericWipe(FSTestCase):
         self.assertEqual(fs_type, b"")
 
         # now do the wipe all in a one step
-        if self._vfat_version >= LooseVersion("4.2"):
+        if self._vfat_version >= Version("4.2"):
             ret = utils.run("mkfs.vfat -I %s >/dev/null 2>&1 --mbr=n" % self.loop_dev)
         else:
             ret = utils.run("mkfs.vfat -I %s >/dev/null 2>&1" % self.loop_dev)
@@ -220,7 +232,7 @@ class TestClean(FSTestCase):
         succ = BlockDev.fs_clean(self.loop_dev)
         self.assertTrue(succ)
 
-        ret = run("pvcreate -ff -y %s >/dev/null 2>&1" % self.loop_dev)
+        ret = run("pvcreate -ff -y %s --config \"devices {use_devicesfile = 0}\" >/dev/null 2>&1" % self.loop_dev)
         self.assertEqual(ret, 0)
 
         succ = BlockDev.fs_clean(self.loop_dev)
@@ -233,7 +245,7 @@ class TestClean(FSTestCase):
 
         # vfat has multiple signatures on the device so it allows us to test
         # that clean removes all signatures
-        if self._vfat_version >= LooseVersion("4.2"):
+        if self._vfat_version >= Version("4.2"):
             ret = utils.run("mkfs.vfat -I %s >/dev/null 2>&1 --mbr=n" % self.loop_dev)
         else:
             ret = utils.run("mkfs.vfat -I %s >/dev/null 2>&1" % self.loop_dev)
@@ -321,7 +333,7 @@ class ExtTestWipe(FSTestCase):
         with self.assertRaises(GLib.GError):
             wipe_function(self.loop_dev)
 
-        run("pvcreate -ff -y %s >/dev/null" % self.loop_dev)
+        run("pvcreate -ff -y %s --config \"devices {use_devicesfile = 0}\" >/dev/null" % self.loop_dev)
 
         # LVM PV signature, not an ext4 file system
         with self.assertRaises(GLib.GError):
@@ -360,11 +372,6 @@ class ExtTestCheck(FSTestCase):
         succ = check_function(self.loop_dev, None)
         self.assertTrue(succ)
 
-        # mounted, but can be checked
-        with mounted(self.loop_dev, self.mount_dir):
-            succ = check_function(self.loop_dev, None)
-            self.assertTrue(succ)
-
         succ = check_function(self.loop_dev, None)
         self.assertTrue(succ)
 
@@ -395,10 +402,6 @@ class ExtTestRepair(FSTestCase):
         succ = repair_function(self.loop_dev, True, None)
         self.assertTrue(succ)
 
-        with mounted(self.loop_dev, self.mount_dir):
-            with self.assertRaises(GLib.GError):
-                repair_function(self.loop_dev, False, None)
-
         succ = repair_function(self.loop_dev, False, None)
         self.assertTrue(succ)
 
@@ -611,7 +614,7 @@ class XfsTestWipe(FSTestCase):
         with self.assertRaises(GLib.GError):
             BlockDev.fs_xfs_wipe(self.loop_dev)
 
-        run("pvcreate -ff -y %s >/dev/null" % self.loop_dev)
+        run("pvcreate -ff -y %s --config \"devices {use_devicesfile = 0}\" >/dev/null" % self.loop_dev)
 
         # LVM PV signature, not an xfs file system
         with self.assertRaises(GLib.GError):
@@ -677,12 +680,7 @@ class XfsGetInfo(FSTestCase):
         succ = BlockDev.fs_xfs_mkfs(self.loop_dev, None)
         self.assertTrue(succ)
 
-        with six.assertRaisesRegex(self, GLib.GError, "not mounted"):
-            fi = BlockDev.fs_xfs_get_info(self.loop_dev)
-
-        with mounted(self.loop_dev, self.mount_dir):
-            fi = BlockDev.fs_xfs_get_info(self.loop_dev)
-
+        fi = BlockDev.fs_xfs_get_info(self.loop_dev)
         self.assertEqual(fi.block_size, 4096)
         self.assertEqual(fi.block_count, 500 * 1024**2 / 4096)
         self.assertEqual(fi.label, "")
@@ -728,15 +726,15 @@ class XfsResize(FSTestCase):
     _loop_size = 500 * 1024**2
 
     def _destroy_lvm(self):
-        run("vgremove --yes libbd_fs_tests >/dev/null 2>&1")
-        run("pvremove --yes %s >/dev/null 2>&1" % self.loop_dev)
+        run("vgremove --yes libbd_fs_tests --config \"devices {use_devicesfile = 0}\" >/dev/null 2>&1")
+        run("pvremove --yes %s --config \"devices {use_devicesfile = 0}\" >/dev/null 2>&1" % self.loop_dev)
 
     def test_xfs_resize(self):
         """Verify that it is possible to resize an xfs file system"""
 
-        run("pvcreate -ff -y %s >/dev/null 2>&1" % self.loop_dev)
-        run("vgcreate -s10M libbd_fs_tests %s >/dev/null 2>&1" % self.loop_dev)
-        run("lvcreate -n xfs_test -L350M libbd_fs_tests >/dev/null 2>&1")
+        run("pvcreate -ff -y %s --config \"devices {use_devicesfile = 0}\" >/dev/null 2>&1" % self.loop_dev)
+        run("vgcreate -s10M libbd_fs_tests %s --config \"devices {use_devicesfile = 0}\" >/dev/null 2>&1" % self.loop_dev)
+        run("lvcreate -n xfs_test -L350M libbd_fs_tests --config \"devices {use_devicesfile = 0}\" >/dev/null 2>&1")
         self.addCleanup(self._destroy_lvm)
         lv = "/dev/libbd_fs_tests/xfs_test"
 
@@ -760,12 +758,12 @@ class XfsResize(FSTestCase):
 
         # (still) impossible to shrink an XFS file system
         xfs_version = _get_xfs_version()
-        if xfs_version < LooseVersion("5.1.12"):
+        if xfs_version < Version("5.1.12"):
             with mounted(lv, self.mount_dir):
                 with self.assertRaises(GLib.GError):
                     succ = BlockDev.fs_resize(lv, 40 * 1024**2)
 
-        run("lvresize -L400M libbd_fs_tests/xfs_test >/dev/null 2>&1")
+        run("lvresize -L400M libbd_fs_tests/xfs_test --config \"devices {use_devicesfile = 0}\" >/dev/null 2>&1")
         # should grow
         with mounted(lv, self.mount_dir):
             succ = BlockDev.fs_xfs_resize(self.mount_dir, 0, None)
@@ -775,7 +773,7 @@ class XfsResize(FSTestCase):
         self.assertTrue(fi)
         self.assertEqual(fi.block_size * fi.block_count, 400 * 1024**2)
 
-        run("lvresize -L450M libbd_fs_tests/xfs_test >/dev/null 2>&1")
+        run("lvresize -L450M libbd_fs_tests/xfs_test --config \"devices {use_devicesfile = 0}\" >/dev/null 2>&1")
         # should grow just to 430 MiB
         with mounted(lv, self.mount_dir):
             succ = BlockDev.fs_xfs_resize(self.mount_dir, 430 * 1024**2 / fi.block_size, None)
@@ -843,7 +841,7 @@ class VfatTestWipe(FSTestCase):
         with self.assertRaises(GLib.GError):
             BlockDev.fs_vfat_wipe(self.loop_dev)
 
-        run("pvcreate -ff -y %s >/dev/null" % self.loop_dev)
+        run("pvcreate -ff -y %s --config \"devices {use_devicesfile = 0}\" >/dev/null" % self.loop_dev)
 
         # LVM PV signature, not an vfat file system
         with self.assertRaises(GLib.GError):
@@ -948,6 +946,141 @@ class VfatResize(FSTestCase):
         succ = BlockDev.fs_vfat_resize(self.loop_dev, 0)
         self.assertTrue(succ)
 
+
+class ExfatTestCase(FSTestCase):
+    def setUp(self):
+        if not self.exfat_avail:
+            self.skipTest("skipping exFAT: not available")
+
+        super(ExfatTestCase, self).setUp()
+
+class ExfatTestMkfs(ExfatTestCase):
+    def test_exfat_mkfs(self):
+        """Verify that it is possible to create a new exfat file system"""
+
+        with self.assertRaises(GLib.GError):
+            BlockDev.fs_exfat_mkfs("/non/existing/device", None)
+
+        succ = BlockDev.fs_exfat_mkfs(self.loop_dev, None)
+        self.assertTrue(succ)
+
+        # just try if we can mount the file system
+        with mounted(self.loop_dev, self.mount_dir):
+            pass
+
+        # check the fstype
+        fstype = BlockDev.fs_get_fstype(self.loop_dev)
+        self.assertEqual(fstype, "exfat")
+
+        BlockDev.fs_wipe(self.loop_dev, True)
+
+class ExfatMkfsWithLabel(ExfatTestCase):
+    def test_exfat_mkfs_with_label(self):
+        """Verify that it is possible to create an exfat file system with label"""
+
+        ea = BlockDev.ExtraArg.new("-n", "test_label")
+        succ = BlockDev.fs_exfat_mkfs(self.loop_dev, [ea])
+        self.assertTrue(succ)
+
+        fi = BlockDev.fs_exfat_get_info(self.loop_dev)
+        self.assertTrue(fi)
+        self.assertEqual(fi.label, "test_label")
+
+class ExfatTestWipe(ExfatTestCase):
+    def test_exfat_wipe(self):
+        """Verify that it is possible to wipe an exfat file system"""
+
+        succ = BlockDev.fs_exfat_mkfs(self.loop_dev, None)
+        self.assertTrue(succ)
+
+        succ = BlockDev.fs_exfat_wipe(self.loop_dev)
+        self.assertTrue(succ)
+
+        # already wiped, should fail this time
+        with self.assertRaises(GLib.GError):
+            BlockDev.fs_exfat_wipe(self.loop_dev)
+
+        run("pvcreate -ff -y %s --config \"devices {use_devicesfile = 0}\" >/dev/null" % self.loop_dev)
+
+        # LVM PV signature, not an exfat file system
+        with self.assertRaises(GLib.GError):
+            BlockDev.fs_exfat_wipe(self.loop_dev)
+
+        BlockDev.fs_wipe(self.loop_dev, True)
+
+        run("mkfs.ext2 -F %s >/dev/null 2>&1" % self.loop_dev)
+
+        # ext2, not an exfat file system
+        with self.assertRaises(GLib.GError):
+            BlockDev.fs_exfat_wipe(self.loop_dev)
+
+        BlockDev.fs_wipe(self.loop_dev, True)
+
+class ExfatTestCheck(ExfatTestCase):
+    def test_exfat_check(self):
+        """Verify that it is possible to check an exfat file system"""
+
+        succ = BlockDev.fs_exfat_mkfs(self.loop_dev, None)
+        self.assertTrue(succ)
+
+        succ = BlockDev.fs_exfat_check(self.loop_dev, None)
+        self.assertTrue(succ)
+
+class ExfatTestRepair(ExfatTestCase):
+    def test_exfat_repair(self):
+        """Verify that it is possible to repair an exfat file system"""
+
+        succ = BlockDev.fs_exfat_mkfs(self.loop_dev, None)
+        self.assertTrue(succ)
+
+        succ = BlockDev.fs_exfat_repair(self.loop_dev, None)
+        self.assertTrue(succ)
+
+class ExfatGetInfo(ExfatTestCase):
+    def test_exfat_get_info(self):
+        """Verify that it is possible to get info about an exfat file system"""
+
+        succ = BlockDev.fs_exfat_mkfs(self.loop_dev, None)
+        self.assertTrue(succ)
+
+        fi = BlockDev.fs_exfat_get_info(self.loop_dev)
+        self.assertTrue(fi)
+        self.assertEqual(fi.label, "")
+        # should be an non-empty string
+        self.assertTrue(fi.uuid)
+        self.assertGreater(fi.sector_size, 0)
+        self.assertGreater(fi.sector_count, 0)
+        self.assertGreater(fi.cluster_count, 0)
+
+class ExfatSetLabel(ExfatTestCase):
+    def test_exfat_set_label(self):
+        """Verify that it is possible to set label of an exfat file system"""
+
+        succ = BlockDev.fs_exfat_mkfs(self.loop_dev, None)
+        self.assertTrue(succ)
+
+        fi = BlockDev.fs_exfat_get_info(self.loop_dev)
+        self.assertTrue(fi)
+        self.assertEqual(fi.label, "")
+
+        succ = BlockDev.fs_exfat_set_label(self.loop_dev, "test_label")
+        self.assertTrue(succ)
+        fi = BlockDev.fs_exfat_get_info(self.loop_dev)
+        self.assertTrue(fi)
+        self.assertEqual(fi.label, "test_label")
+
+        succ = BlockDev.fs_exfat_set_label(self.loop_dev, "test_label2")
+        self.assertTrue(succ)
+        fi = BlockDev.fs_exfat_get_info(self.loop_dev)
+        self.assertTrue(fi)
+        self.assertEqual(fi.label, "test_label2")
+
+        succ = BlockDev.fs_exfat_set_label(self.loop_dev, "")
+        self.assertTrue(succ)
+        fi = BlockDev.fs_exfat_get_info(self.loop_dev)
+        self.assertTrue(fi)
+        self.assertEqual(fi.label, "")
+
 class CanResizeRepairCheckLabel(FSTestCase):
     def test_can_resize(self):
         """Verify that tooling query works for resize"""
@@ -1341,6 +1474,12 @@ class GenericCheck(FSTestCase):
             self.skipTest("skipping NTFS: not available")
         self._test_generic_check(mkfs_function=BlockDev.fs_ntfs_mkfs)
 
+    def test_exfat_generic_check(self):
+        """Test generic check function with an exfat file system"""
+        if not self.exfat_avail:
+            self.skipTest("skipping exFAT: not available")
+        self._test_generic_check(mkfs_function=BlockDev.fs_exfat_mkfs)
+
 class GenericRepair(FSTestCase):
     _loop_size = 500 * 1024**2
 
@@ -1369,6 +1508,12 @@ class GenericRepair(FSTestCase):
             self.skipTest("skipping NTFS: not available")
         self._test_generic_repair(mkfs_function=BlockDev.fs_ntfs_mkfs)
 
+    def test_exfat_generic_repair(self):
+        """Test generic repair function with an exfat file system"""
+        if not self.exfat_avail:
+            self.skipTest("skipping exFAT: not available")
+        self._test_generic_repair(mkfs_function=BlockDev.fs_exfat_mkfs)
+
 class GenericSetLabel(FSTestCase):
     _loop_size = 500 * 1024**2
 
@@ -1397,6 +1542,12 @@ class GenericSetLabel(FSTestCase):
             self.skipTest("skipping NTFS: not available")
         self._test_generic_set_label(mkfs_function=BlockDev.fs_ntfs_mkfs)
 
+    def test_exfat_generic_set_label(self):
+        """Test generic set_label function with a exfat file system"""
+        if not self.exfat_avail:
+            self.skipTest("skipping exFAT: not available")
+        self._test_generic_set_label(mkfs_function=BlockDev.fs_exfat_mkfs)
+
 class GenericResize(FSTestCase):
     _loop_size = 500 * 1024**2
 
@@ -1465,7 +1616,7 @@ class GenericResize(FSTestCase):
     def test_vfat_generic_resize(self):
         """Test generic resize function with a vfat file system"""
         def mkfs_vfat(device, options=None):
-            if self._vfat_version >= LooseVersion("4.2"):
+            if self._vfat_version >= Version("4.2"):
                 if options:
                     return BlockDev.fs_vfat_mkfs(device, options + [BlockDev.ExtraArg.new("--mbr=n", "")])
                 else:
@@ -1476,15 +1627,15 @@ class GenericResize(FSTestCase):
         self._test_generic_resize(mkfs_function=mkfs_vfat)
 
     def _destroy_lvm(self):
-        run("vgremove --yes libbd_fs_tests >/dev/null 2>&1")
-        run("pvremove --yes %s >/dev/null 2>&1" % self.loop_dev)
+        run("vgremove --yes libbd_fs_tests --config \"devices {use_devicesfile = 0}\" >/dev/null 2>&1")
+        run("pvremove --yes %s --config \"devices {use_devicesfile = 0}\" >/dev/null 2>&1" % self.loop_dev)
 
     def test_xfs_generic_resize(self):
         """Test generic resize function with an xfs file system"""
 
-        run("pvcreate -ff -y %s >/dev/null 2>&1" % self.loop_dev)
-        run("vgcreate -s10M libbd_fs_tests %s >/dev/null 2>&1" % self.loop_dev)
-        run("lvcreate -n xfs_test -L350M libbd_fs_tests >/dev/null 2>&1")
+        run("pvcreate -ff -y %s --config \"devices {use_devicesfile = 0}\" >/dev/null 2>&1" % self.loop_dev)
+        run("vgcreate -s10M libbd_fs_tests %s --config \"devices {use_devicesfile = 0}\" >/dev/null 2>&1" % self.loop_dev)
+        run("lvcreate -n xfs_test -L350M libbd_fs_tests --config \"devices {use_devicesfile = 0}\" >/dev/null 2>&1")
         self.addCleanup(self._destroy_lvm)
 
         lv = "/dev/libbd_fs_tests/xfs_test"
@@ -1512,12 +1663,12 @@ class GenericResize(FSTestCase):
 
         # (still) impossible to shrink an XFS file system
         xfs_version = _get_xfs_version()
-        if xfs_version < LooseVersion("5.1.12"):
+        if xfs_version < Version("5.1.12"):
             with mounted(lv, self.mount_dir):
                 with self.assertRaises(GLib.GError):
                     succ = BlockDev.fs_resize(lv, 40 * 1024**2)
 
-        run("lvresize -L400M libbd_fs_tests/xfs_test >/dev/null 2>&1")
+        run("lvresize -L400M libbd_fs_tests/xfs_test --config \"devices {use_devicesfile = 0}\" >/dev/null 2>&1")
         # should grow
         with mounted(lv, self.mount_dir):
             succ = BlockDev.fs_resize(lv, 0)
@@ -1527,7 +1678,7 @@ class GenericResize(FSTestCase):
         self.assertTrue(fi)
         self.assertEqual(fi.block_size * fi.block_count, 400 * 1024**2)
 
-        run("lvresize -L450M libbd_fs_tests/xfs_test >/dev/null 2>&1")
+        run("lvresize -L450M libbd_fs_tests/xfs_test --config \"devices {use_devicesfile = 0}\" >/dev/null 2>&1")
         # should grow just to 430 MiB
         with mounted(lv, self.mount_dir):
             succ = BlockDev.fs_resize(lv, 430 * 1024**2)
@@ -1545,7 +1696,22 @@ class GenericResize(FSTestCase):
             fi = BlockDev.fs_xfs_get_info(lv)
         self.assertTrue(fi)
         self.assertEqual(fi.block_size * fi.block_count, 450 * 1024**2)
+        # self.assertEqual(fi.block_size * fi.block_count, 90 * 1024**2)
 
+    def test_exfat_generic_resize(self):
+        """Test generic resize function with an exfat file system"""
+        if not self.exfat_avail:
+            self.skipTest("skipping exFAT: not available")
+
+        # clean the device
+        succ = BlockDev.fs_clean(self.loop_dev)
+
+        succ = BlockDev.fs_exfat_mkfs(self.loop_dev, None)
+        self.assertTrue(succ)
+
+        # no resize support for exFAT
+        with six.assertRaisesRegex(self, GLib.GError, "Resizing filesystem 'exfat' is not supported."):
+            BlockDev.fs_resize(self.loop_dev, 80 * 1024**2)
 
 class FSFreezeTest(FSTestCase):
     _loop_size = 500 * 1024**2
