@@ -9,9 +9,13 @@ import shutil
 import subprocess
 import time
 from contextlib import contextmanager
-from distutils.version import LooseVersion
+from packaging.version import Version
 
 from utils import create_sparse_tempfile, create_lio_device, delete_lio_device, fake_utils, fake_path, TestTags, tag_test, run_command
+
+import gi
+gi.require_version("BlockDev", "2.0")
+
 from gi.repository import BlockDev, GLib
 
 
@@ -50,7 +54,7 @@ class LVMTestCase(unittest.TestCase):
         m = re.search(r"LVM version:\s+([\d\.]+)", out)
         if not m or len(m.groups()) != 1:
             raise RuntimeError("Failed to determine LVM version from: %s" % out)
-        return LooseVersion(m.groups()[0])
+        return Version(m.groups()[0])
 
 
 class LvmNoDevTestCase(LVMTestCase):
@@ -264,6 +268,24 @@ class LvmPVonlyTestCase(LVMTestCase):
 
     _sparse_size = 1024**3
 
+    @classmethod
+    def setUpClass(cls):
+        super().setUpClass()
+
+        try:
+            BlockDev.lvm_set_global_config("devices {use_devicesfile = 0}")
+        except:
+            pass
+
+    @classmethod
+    def tearDownClass(cls):
+        super().tearDownClass()
+
+        try:
+            BlockDev.lvm_set_global_config("")
+        except:
+            pass
+
     # :TODO:
     #     * test pvmove (must create two PVs, a VG, a VG and some data in it
     #       first)
@@ -350,9 +372,15 @@ class LvmTestPVresize(LvmPVonlyTestCase):
         succ = BlockDev.lvm_pvresize(self.loop_dev, 200 * 1024**2, None)
         self.assertTrue(succ)
 
+        info = BlockDev.lvm_pvinfo(self.loop_dev)
+        self.assertEqual(info.pv_size, 200 * 1024**2)
+
         succ = BlockDev.lvm_pvresize(self.loop_dev, 200 * 1024**3, None)
         self.assertTrue(succ)
 
+        info = BlockDev.lvm_pvinfo(self.loop_dev)
+        self.assertEqual(info.pv_size, 200 * 1024**3)
+
 class LvmTestPVscan(LvmPVonlyTestCase):
     def test_pvscan(self):
         """Verify that pvscan runs without issues with cache or without"""
@@ -830,6 +858,10 @@ class LvmTestLVresize(LvmPVVGLVTestCase):
         succ = BlockDev.lvm_lvdeactivate("testVG", "testLV", None)
         self.assertTrue(succ)
 
+         # try to shrink when deactivated
+        succ = BlockDev.lvm_lvresize("testVG", "testLV", 400 * 1024**2, None)
+        self.assertTrue(succ)
+
 class LvmTestLVrename(LvmPVVGLVTestCase):
     def test_lvrename(self):
         """Verify that it's possible to rename an LV"""
@@ -1330,7 +1362,7 @@ class LvmPVVGcachedLVpoolTestCase(LvmPVVGLVTestCase):
         self.assertTrue(succ)
 
         lvm_version = self._get_lvm_version()
-        if lvm_version < LooseVersion("2.03.06"):
+        if lvm_version < Version("2.03.06"):
             cpool_name = "testCache"
         else:
             # since 2.03.06 LVM adds _cpool suffix to the cache pool after attaching it
@@ -1491,7 +1523,7 @@ class LVMVDOTest(LVMTestCase):
                 raise unittest.SkipTest("cannot load VDO kernel module, skipping.")
 
         lvm_version = cls._get_lvm_version()
-        if lvm_version < LooseVersion("2.3.07"):
+        if lvm_version < Version("2.3.07"):
             raise unittest.SkipTest("LVM version 2.3.07 or newer needed for LVM VDO.")
 
         super().setUpClass()
@@ -1713,5 +1745,7 @@ class LvmConfigTestPvremove(LvmPVonlyTestCase):
         self.assertTrue(succ)
 
         BlockDev.lvm_set_global_config("")
-        succ = BlockDev.lvm_pvremove(self.loop_dev)
+
+        ea = BlockDev.ExtraArg.new("--config", "devices {use_devicesfile = 0}")
+        succ = BlockDev.lvm_pvremove(self.loop_dev, extra=[ea])
         self.assertTrue(succ)
