@@ -7,12 +7,12 @@ import datetime
 import os
 import re
 import six
+import shutil
 import subprocess
 import sys
 import unittest
 import yaml
 
-from distutils.spawn import find_executable
 
 LIBDIRS = 'src/utils/.libs:src/plugins/.libs:src/plugins/fs/.libs:src/lib/.libs'
 GIDIR = 'src/lib'
@@ -210,6 +210,10 @@ if __name__ == '__main__':
         sys.path.append(projdir)
         sys.path.append(os.path.join(projdir, 'src/python'))
 
+    if not args.installed:
+        import gi.overrides
+        gi.overrides.__path__.insert(0, os.path.join(projdir, 'src/python/gi/overrides'))
+
     start_time = datetime.datetime.now().strftime("%Y-%m-%d %H:%M:%S")
 
     loader = unittest.defaultTestLoader
@@ -278,7 +282,7 @@ if __name__ == '__main__':
     result = unittest.TextTestRunner(verbosity=2, failfast=args.stop).run(suite)
 
     # dump cropped journal to log file
-    if find_executable('journalctl'):
+    if shutil.which('journalctl'):
         with open('journaldump.log', 'w') as outfile:
             subprocess.call(['journalctl', '-S', start_time], stdout=outfile)
 
